Search.setIndex({docnames:["index","modules"],envversion:{"sphinx.domains.c":1,"sphinx.domains.changeset":1,"sphinx.domains.cpp":1,"sphinx.domains.javascript":1,"sphinx.domains.math":2,"sphinx.domains.python":1,"sphinx.domains.rst":1,"sphinx.domains.std":1,sphinx:55},filenames:["index.rst","modules.rst"],objects:{etlstat:{database:[1,0,0,"-"],extractor:[1,0,0,"-"],log:[1,0,0,"-"],text:[1,0,0,"-"]}},objnames:{"0":["py","module","Python module"]},objtypes:{"0":"py:module"},terms:{content:0,etlstat:0,index:0,page:0,search:0},titles:["Welcome to pyaxis documentation!","etlstat"],titleterms:{document:0,etlstat:1,indic:0,modul:0,pyaxi:0,tabl:0,welcom:0}})